@@ -7,18 +7,18 @@ use Carp;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(tidy_xml object_to_xml xml_to_object simple_to_xml xml_to_simple check_xml);
-
+our %EXPORT_TAGS = (all => [@EXPORT_OK]);
 =head1 NAME
 
 XML::MyXML - A simple XML module
 
 =head1 VERSION
 
-Version 0.091
+Version 0.092
 
 =cut
 
-our $VERSION = '0.091';
+our $VERSION = '0.092';
 
 =head1 SYNOPSIS
 
@@ -90,9 +90,10 @@ Returns the XML string in a tidy format (with tabs & newlines)
 sub tidy_xml {
 	my $xml = shift;
 	if ($xml eq 'XML::MyXML') { $xml = shift; }
+	my $flags = shift || {};
 
 	my $object = &xml_to_object($xml);
-	&_tidy_object($object);
+	&_tidy_object($object, undef, $flags);
 	return &object_to_xml($object);
 }
 
@@ -101,7 +102,7 @@ sub tidy_xml {
 
 Creates an 'XML::MyXML::Object' object from the raw XML provided
 
-If called with the C<file> flag (like this: C<< xml_to_object($filename, { file => 1 }) >>), then the function's first parameter, instead of being an XML string, should be the path to a file that contains the XML document.
+If called with the C<file> flag (like this: C<< xml_to_object($filename, { file => 1 }) >>), then the function's first parameter, instead of being an XML string, should be the path to a file that contains the XML document. If called with the C<soft> flag, the function will return C<undef> instead of dying (in case of error).
 
 =cut
 
@@ -110,10 +111,10 @@ sub xml_to_object {
 	if ($xml eq 'XML::MyXML') { $xml = shift; }
 	my $flags = (@_ and defined $_[0]) ? $_[0] : {};
 
-	my $jk = $flags->{'jk'}; # just checking xml
+	my $soft = $flags->{'soft'}; # soft = 'don't die if can't parse, just return undef'
 
 	if ($flags->{'file'}) {
-		open FILE, $xml or confess "Error: The file '$xml' could not be opened for reading.";
+		open FILE, $xml or do { confess "Error: The file '$xml' could not be opened for reading." unless $soft; return undef; };
 		$xml = join '', <FILE>;
 		close FILE;
 	}
@@ -128,7 +129,7 @@ sub xml_to_object {
 		my $init_ws = 1;
 		foreach my $el (@els) {
 			if ($el =~ /^<!--/) {
-				if ($el !~ /-->$/) { confess "Error: unclosed XML comment block - '$el'" unless $jk; return 0; }
+				if ($el !~ /-->$/) { confess "Error: unclosed XML comment block - '$el'" unless $soft; return undef; }
 				undef $el;
 				next;
 			} elsif ($init_ws) {
@@ -140,7 +141,7 @@ sub xml_to_object {
 			}
 		}
 		@els = grep { defined $_ } @els;
-		if (! @els) { confess "Error: No elements in XML document" unless $jk; return 0; }
+		if (! @els) { confess "Error: No elements in XML document" unless $soft; return undef; }
 	}
 	my @stack;
 	my $object = { content => [] };
@@ -149,11 +150,11 @@ sub xml_to_object {
 		if ($el =~ /^<\?[^>]*\?>$/) {
 			next;
 		} elsif ($el =~ /^<\/?>$/) {
-			confess "Error: Strange element: '$el'" unless $jk; return 0;
+			confess "Error: Strange element: '$el'" unless $soft; return undef;
 		} elsif ($el =~ /^<\/[^\s>]+>$/) {
 			my ($element) = $el =~ /^<\/(\S+)>$/g;
-			if (! length($element)) { confess "Error: Strange element: '$el'" unless $jk; return 0; }
-			if ($stack[$#stack]->{'element'} ne $element) { confess "Error: Incompatible stack element: stack='".$stack[$#stack]->{'element'}."' element='$el'" unless $jk; return 0; }
+			if (! length($element)) { confess "Error: Strange element: '$el'" unless $soft; return undef; }
+			if ($stack[$#stack]->{'element'} ne $element) { confess "Error: Incompatible stack element: stack='".$stack[$#stack]->{'element'}."' element='$el'" unless $soft; return undef; }
 			my $stackentry = pop @stack;
 			if ($#{$stackentry->{'content'}} == -1) {
 				delete $stackentry->{'content'};
@@ -161,7 +162,7 @@ sub xml_to_object {
 			$pointer = $stackentry->{'parent'};
 		} elsif ($el =~ /^<[^>]+\/>$/) {
 			my ($element) = $el =~ /^<([^\s>\/]+)/g;
-			if (! length($element)) { confess "Error: Strange element: '$el'" unless $jk; return 0; }
+			if (! length($element)) { confess "Error: Strange element: '$el'" unless $soft; return undef; }
 			my $elementmeta = quotemeta($element);
 			$el =~ s/^<$elementmeta//;
 			$el =~ s/\/>$//;
@@ -171,7 +172,7 @@ sub xml_to_object {
 			my %attr;
 			foreach my $attr (@attrs) {
 				my ($name, undef, $value) = $attr =~ /^(\S+?)=(['"])(.*?)\2$/g;
-				if (! length($name) or ! defined($value)) { confess "Error: Strange attribute: '$attr'" unless $jk; return 0; }
+				if (! length($name) or ! defined($value)) { confess "Error: Strange attribute: '$attr'" unless $soft; return undef; }
 				$attr{$name} = &_decode($value);
 			}
 			my $entry = { element => $element, attrs => \%attr, parent => $pointer };
@@ -179,7 +180,7 @@ sub xml_to_object {
 			push @{$pointer->{'content'}}, $entry;
 		} elsif ($el =~ /^<[^\s>\/][^>]*>$/) {
 			my ($element) = $el =~ /^<([^\s>]+)/g;
-			if (! length($element)) { confess "Error: Strange element: '$el'" unless $jk; return 0; }
+			if (! length($element)) { confess "Error: Strange element: '$el'" unless $soft; return undef; }
 			my $elementmeta = quotemeta($element);
 			$el =~ s/^<$elementmeta//;
 			$el =~ s/>$//;
@@ -189,7 +190,7 @@ sub xml_to_object {
 			my %attr;
 			foreach my $attr (@attrs) {
 				my ($name, undef, $value) = $attr =~ /^(\S+?)=(['"])(.*?)\2$/g;
-				if (! length($name) or ! defined($value)) { confess "Error: Strange attribute: '$attr'" unless $jk; return 0; }
+				if (! length($name) or ! defined($value)) { confess "Error: Strange attribute: '$attr'" unless $soft; return undef; }
 				$attr{$name} = &_decode($value);
 			}
 			my $entry = { element => $element, attrs => \%attr, content => [], parent => $pointer };
@@ -202,9 +203,10 @@ sub xml_to_object {
 			bless $entry, 'XML::MyXML::Object';
 			push @{$pointer->{'content'}}, $entry;
 		} else {
-			confess "Error: Strange element: '$el'" unless $jk; return 0;
+			confess "Error: Strange element: '$el'" unless $soft; return undef;
 		}
 	}
+	if (@stack) { confess "Error: some elements have not been closed in XML" unless $soft; return undef; }
 	$object = $object->{'content'}[0];
 	$object->{'parent'} = undef;
 	return $object;
@@ -253,6 +255,9 @@ sub object_to_xml {
 sub _tidy_object {
 	my $object = shift;
 	my $tabs = shift || 0;
+	my $flags = shift || {};
+
+	if (! exists $flags->{'indentstring'}) { $flags->{'indentstring'} = "\t" }
 
 	if (! defined $object->{'content'} or ! @{$object->{'content'}}) { return; }
 	my $hastext;
@@ -273,13 +278,13 @@ sub _tidy_object {
 	@children = @{$object->{'content'}};
 	$object->{'content'} = [];
 	for my $i (0..$#children) {
-		push @{$object->{'content'}}, { value => "\n".("\t"x($tabs+1)), parent => $object };
+		push @{$object->{'content'}}, { value => "\n".($flags->{'indentstring'}x($tabs+1)), parent => $object };
 		push @{$object->{'content'}}, $children[$i];
 	}
-	push @{$object->{'content'}}, { value => "\n".("\t"x($tabs)), parent => $object };
+	push @{$object->{'content'}}, { value => "\n".($flags->{'indentstring'}x($tabs)), parent => $object };
 	
 	for my $i (0..$#{$object->{'content'}}) {
-		&_tidy_object($object->{'content'}[$i], $tabs+1);
+		&_tidy_object($object->{'content'}[$i], $tabs+1, $flags);
 	}
 }
 
@@ -364,9 +369,11 @@ You can use flags like this: S<C<< &xml_to_simple($raw_xml, {strip => 1, interna
 
 If called with the 'C<internal>' flag, then the hashref returned will be the first value of the hashref that would otherwise be returned, i.e. a hashref created only by the contents of the top element in the XML and which doesn't contain information about the top-level tag (See L</SYNOPSIS> section for an example)
 
-If called with the 'C<strip>' flag, then all text contents will be stripped of possible beginning and/or ending whitespace.
+If called with the 'C<strip>' flag, all text contents will be stripped of possible beginning and/or ending whitespace.
 
-If called with the 'C<file>' flag, then the function's first parameter, instead of being an XML string, should be the path to a file that holds the XML document.
+If called with the 'C<file>' flag, then the function's first parameter should be the path to an XML file, instead of being an XML string.
+
+If called with the 'C<soft>' flag, the function will return undef instead of dying (in case of error)
 
 =cut
 
@@ -376,15 +383,9 @@ sub xml_to_simple {
 
 	if (ref $flags ne 'HASH') { confess "Error: This method of setting flags is deprecated in XML::MyXML v0.083 - check module's documentation for the new way"; }
 
-	if ($flags->{'file'}) {
-		open FILE, $xml or confess "Error: The file '$xml' could not be opened for reading.";
-		$xml = join '', <FILE>;
-		close FILE;
-	}
+	my $object = &xml_to_object($xml, $flags);
 
-	my $object = &xml_to_object($xml);
-
-	return $object->simplify($flags);
+	return defined $object ? $object->simplify($flags) : $object;
 }
 
 sub _objectarray_to_simple {
@@ -417,13 +418,17 @@ sub _objectarray_to_simple {
 
 =head2 check_xml($raw_xml)
 
-Returns 1 if the $raw_xml string is valid XML (valid enough to be used by this module), and 0 otherwise
+Returns 1 if the $raw_xml string is valid XML (valid enough to be used by this module), and 0 otherwise. Can be used with the C<file> flag. (See L<xml_to_simple|/xml_to_simple($raw_xml)> for meaning and example of this flag)
 
 =cut
 
 sub check_xml {
 	my $xml = shift;
-	return 1 if &xml_to_object($xml, { jk => 1 }); # jk = 'just checking'
+	my $flags = (@_ and defined $_[0]) ? $_[0] : {};
+
+	if (ref $flags ne 'HASH') { confess "Error: This method of setting flags is deprecated in XML::MyXML v0.083 - check module's documentation for the new way"; }
+
+	return 1 if &xml_to_object($xml, { %$flags, soft => 1 }); # soft = 'don't die if can't parse, just return undef'
 	return 0;
 }
 
@@ -461,7 +466,7 @@ sub children {
 
 =head2 $obj->path("subtag1/subsubtag2/.../subsubsubtagX")
 
-Returns the element specified by the path as an XML::MyXML::Object object. When there are more than one tags with the specified name in the last step of the path, it will return all of them as an array.
+Returns the element specified by the path as an XML::MyXML::Object object. When there are more than one tags with the specified name in the last step of the path, it will return all of them as an array. In scalar context will only return the first one.
 
 =cut
 
@@ -483,7 +488,7 @@ sub path {
 
 When the element represented by the $obj object has only text contents, returns those contents as a string. If the $obj element has no contents, value will return an empty string.
 
-Can be used with the 'strip' flag (as in: S<C<< $obj->value({ strip => 1 }) >>>) to strip possible whitespace in the beginning and end of the value.
+Can be used with the C<strip> flag (as in: S<C<< $obj->value({ strip => 1 }) >>>) to strip possible whitespace in the beginning and end of the value.
 
 =cut
 
@@ -513,7 +518,7 @@ sub attr {
 
 =head2 $obj->tag
 
-Returns the tag of the $obj element (after stripping it from namespaces). E.g. if $obj represents an <rss:item> element, $obj->tag will just return 'item'.
+Returns the tag of the $obj element (after stripping it from namespaces). E.g. if $obj represents an <rss:item> element, C<< $obj->tag >> will just return the name iitem'.
 Returns undef if $obj doesn't represent a tag.
 
 =cut
@@ -532,7 +537,7 @@ sub tag {
 
 =head2 $obj->simplify
 
-Returns a very simple hashref, like the one returned with &XML::MyXML::xml_to_simple. Same restrictions and warnings apply. May be called with the 'internal' and/or 'strip' flags (for more on these flags, see this documentation on the 'xml_to_simple' function)
+Returns a very simple hashref, like the one returned with &XML::MyXML::xml_to_simple. Same restrictions and warnings apply. May be called with the C<internal> and/or C<strip> flags (for more on these flags, see this documentation on the L<xml_to_simple|/xml_to_simple($raw_xml)> function)
 
 =cut
 
@@ -548,7 +553,7 @@ sub simplify {
 
 =head2 $obj->to_xml
 
-Returns the XML string of the object, just like calling &object_to_xml( $obj )
+Returns the XML string of the object, just like calling C<&object_to_xml( $obj )>
 
 =cut
 
@@ -560,7 +565,7 @@ sub to_xml {
 
 =head2 $obj->to_tidy_xml
 
-Returns the XML string of the object in tidy form, just like calling &tidy_xml( &object_to_xml( $obj ) )
+Returns the XML string of the object in tidy form, just like calling C<&tidy_xml( &object_to_xml( $obj ) )>
 
 =cut
 
